--[[
Name: LibTalentQuery-1.0
Revision: $Rev: 65353 $
Author: Rich Martel (richmartel@gmail.com)
Documentation: http://wowace.com/wiki/LibTalentQuery-1.0
SVN: http://svn.wowace.com/wowace/trunk/LibTalentQuery-1.0
Description: Library to help with querying unit talents
Dependancies: LibStub, CallbackHandler-1.0
License: LGPL v2.1

Example Usage:
	local TalentQuery = LibStub:GetLibrary("LibTalentQuery-1.0")
	TalentQuery.RegisterCallback(self, "TalentQuery_Ready")
	local spec = {}
	...
	TalentQuery:Query(unit)
	...
	function MyAddon:TalentQuery_Ready(e, unitname)
		for tab = 1, GetNumTalentTabs(true) do
			local treename, _, pointsspent = GetTalentTabInfo(tab, true)
			if (pointsspent > 30) then
				spec[unitname] = treename
			end
		end
		...
	end
]]

local INSPECTDELAY = 1
local MAJOR, MINOR = "LibTalentQuery-1.0", "$Rev: 65353 $"

local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

lib.events = lib.events or LibStub("CallbackHandler-1.0"):New(lib)

lib.frame = lib.frame or CreateFrame("Frame", MAJOR .. "_Frame")
lib.frame:SetScript("OnEvent", function(this, event, ...) lib[event](lib, ...) end)

local inspectingUnitName = nil
local inspectQueue = {}
local lastUpdateTime = 0

function lib:UnitNameServer(unit)
	local name, server = UnitName(unit)
	if name and server and (server ~= "") then
		name = name .. "-" .. server
	end
	return name
end

function lib:UnitId(name)
	local unit
	if (not unit) and (GetNumRaidMembers() > 0) then
		for i = 1, GetNumRaidMembers() do
			if UnitExists("raid" .. i) and (name == self:UnitNameServer("raid" .. i)) then
				unit = "raid" .. i
				break
			end
		end
	end
	if (not unit) and (GetNumPartyMembers() > 0) then
		for i = 1, GetNumPartyMembers() do
			if UnitExists("party" .. i) and (name == self:UnitNameServer("party" .. i)) then
				unit = "party" .. i
				break
			end
		end
	end
	if (not unit) and UnitExists("player") and (name == self:UnitNameServer("player")) then
		unit = "player"
	end
	if (not unit) and UnitExists("focus") and (name == self:UnitNameServer("focus")) then
		unit = "focus"
	end
	if (not unit) and UnitExists("target") and (name == self:UnitNameServer("target")) then
		unit = "target"
	end
	if (not unit) and UnitExists("pettarget") and (name == self:UnitNameServer("pettarget")) then
		unit = "pettarget"
	end
	if (not unit) and UnitExists("targettarget") and (name == self:UnitNameServer("targettarget")) then
		unit = "targettarget"
	end
	if (not unit) and UnitExists("mouseover") and (name == self:UnitNameServer("mouseover")) then
		unit = "mouseover"
	end
	if (not unit) and (GetNumRaidMembers() > 0) then
		for i = 1, GetNumRaidMembers() do
			if UnitExists("raid" .. i .. "target") and (name == self:UnitNameServer("raid" .. i .. "target")) then
				unit = "raid" .. i .. "target"
				break
			end
		end
	end
	if (not unit) and (GetNumPartyMembers() > 0) then
		for i = 1, GetNumPartyMembers() do
			if UnitExists("party" .. i .. "target") and (name == self:UnitNameServer("party" .. i .. "target")) then
				unit = "party" .. i .. "target"
				break
			end
		end
	end
	if (not unit) and (GetNumRaidMembers() > 0) then
		for i = 1, GetNumRaidMembers() do
			if UnitExists("raidpet" .. i .. "target") and (name == self:UnitNameServer("raidpet" .. i .. "target")) then
				unit = "raidpet" .. i .. "target"
				break
			end
		end
	end
	if (not unit) and (GetNumPartyMembers() > 0) then
		for i = 1, GetNumPartyMembers() do
			if UnitExists("partypet" .. i .. "target") and (name == self:UnitNameServer("partypet" .. i .. "target")) then
				unit = "partypet" .. i .. "target"
				break
			end
		end
	end
	return unit
end

function lib:Query(unit)
	if unit and UnitExists(unit) and UnitIsPlayer(unit) then
		inspectQueue[self:UnitNameServer(unit)] = true
		self.frame:SetScript("OnUpdate", function(this, elapsed) return self:OnUpdate(elapsed) end)
	else
		error("Usage: TalentQuery:Query(unit): 'unit' must be a valid player unit.")
	end
end

function lib:CheckInspectQueue()
	local repeatqueuecheck
	if (not inspectingUnitName) then
		for name in pairs(inspectQueue) do
			local unit = self:UnitId(name)
			if unit then
				if UnitIsVisible(unit) and UnitIsConnected(unit) then
					inspectingUnitName = name
					inspectQueue[inspectingUnitName] = nil
					self.frame:RegisterEvent("INSPECT_TALENT_READY")
					NotifyInspect(unit)
					break
				end
			else
				inspectQueue[name] = nil
			end
			repeatqueuecheck = true
		end
	else
		repeatqueuecheck = true
	end
	if (not repeatqueuecheck) then
		self.frame:SetScript("OnUpdate", nil)
	end
end

function lib:OnUpdate(elapsed)
	lastUpdateTime = lastUpdateTime + elapsed
	if (lastUpdateTime > INSPECTDELAY) then
		self:CheckInspectQueue()
		lastUpdateTime = 0
	end
end

hooksecurefunc("NotifyInspect", function(...) return lib:NotifyInspect(...) end)
function lib:NotifyInspect(unit)
	-- If this library did not call NotifyInspect, and we are waiting for a query, then
	-- stop waiting, and put the inspectingUnitName back on the queue.
	if inspectingUnitName and (inspectingUnitName ~= self:UnitNameServer(unit)) then
		self.frame:UnregisterEvent("INSPECT_TALENT_READY")
		inspectQueue[inspectingUnitName] = true
		inspectingUnitName = nil
	end
end

function lib:INSPECT_TALENT_READY()
	if inspectingUnitName then
		local server
		local name = inspectingUnitName
		if string.find(name, "-") then
			name, server = strsplit("-", name)
		end
		self.frame:UnregisterEvent("INSPECT_TALENT_READY")
		self.events:Fire("TalentQuery_Ready", name, server)
		inspectingUnitName = nil
	end
end
